#include "App.h"

#include "../Framework3/GUI/Dialog.h"

BaseApp *app = new App();

void App::resetCamera(){
	camPos = vec3(0.73f, -0.25f, -0.28f);
	wx = 0;
	wy = 0;
}

bool App::onKey(const uint key, const bool pressed){
	if (D3DApp::onKey(key, pressed)) return true;

	if (key >= '1' && key <= '6'){
		int newMode = key - '1';
		if (modesSupported[newMode]){
			mode = newMode;
			errorMsg = NULL;
		} else {
			const char *messages[] = {
				"This mode requires support for DXT1",
				"This mode requires support for DXT1 and DXT5",
				"This mode requires support for DXT1 and DXT5",
				"This mode requires support for ATI1N and ATI2N",
				"This mode requires support for ATI1N and ATI2N",
				"This mode requires support for RGB8",
			};
			errorMsg = messages[newMode];
			errorTime = time;
		}
	}

	return true;
}


bool App::init(){
	speed = 1;
	errorMsg = NULL;

	return true;
}

/*
#define COMPRESSIONATOR_PATH "C:\\Program\\ATI Technologies Inc\\The Compressonator\\TheCompressonator.exe"

bool makeYCbCr(const char *fileName, int downSampleCbCr){
	Image img;
	if (!img.loadImage(fileName)) return false;

	// Make sure it's the right format
	img.unpackImage();
//	img.uncompressImage();
	img.convert(FORMAT_RGB8);
	img.createMipMaps();


	Image imgY, imgCbCr;
	ubyte *dst0 = imgY.create(FORMAT_I8, img.getWidth(), img.getHeight(), img.getDepth(), img.getMipMapCount());
//	ubyte *dst1 = imgCbCr.create(FORMAT_RGB8, img.getWidth(), img.getHeight(), img.getDepth(), img.getMipMapCount());
	ubyte *dst1 = imgCbCr.create(FORMAT_IA8, img.getWidth(), img.getHeight(), img.getDepth(), img.getMipMapCount());

	int nPixels = img.getPixelCount();
	ubyte *src = img.getPixels();
	for (int i = 0; i < nPixels; i++){
		float R = src[0];
		float G = src[1];
		float B = src[2];

		float Y  =  0.299f  * R + 0.587f  * G + 0.114f  * B;
		float Cb = -0.1687f * R - 0.3313f * G + 0.5f    * B + 128;
		float Cr =  0.5f    * R - 0.4187f * G - 0.0813f * B + 128;

		*dst0++ = ubyte(Y);
		*dst1++ = ubyte(Cb);
		*dst1++ = ubyte(Cr);
		//*dst1++ = 0;

		src += 3;
	}

//	imgY.removeMipMaps(0, imgY.getMipMapCount() - 2);
	imgCbCr.removeMipMaps(downSampleCbCr, imgCbCr.getMipMapCount() - downSampleCbCr);
//	imgCbCr.removeMipMaps(downSampleCbCr, imgCbCr.getMipMapCount() - 2 - downSampleCbCr);
//	imgCbCr.removeMipMaps(downSampleCbCr, ALL_MIPMAPS);

	char str[256], cmd[384];

	strcpy(str, fileName);
	char *ext = strrchr(str, '.');

	strcpy(ext, "Y.dds");
	imgY.saveDDS(str);
//	sprintf(cmd, "%s -convert -overwrite \"%s\" \"%s\" +fourCC ATI1", COMPRESSIONATOR_PATH, str, str);
	sprintf(cmd, "%s -convert -overwrite \"%s\" \"%s\" +fourCC DXT1", COMPRESSIONATOR_PATH, str, str);
	char *ch = cmd;
	while (ch = strchr(ch, '/')) *ch = '\\';
	WinExec(cmd, SW_HIDE);

	strcpy(ext, "CbCr.dds");
	imgCbCr.saveDDS(str);
//	sprintf(cmd, "%s -convert -overwrite \"%s\" \"%s\" +fourCC ATI2", COMPRESSIONATOR_PATH, str, str);
	sprintf(cmd, "%s -convert -overwrite \"%s\" \"%s\" +fourCC DXT5", COMPRESSIONATOR_PATH, str, str);
	ch = cmd;
	while (ch = strchr(ch, '/')) *ch = '\\';
	WinExec(cmd, SW_HIDE);

	return true;
}
*/

bool App::load(){
	bool supportsATI1N = (d3d->CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, D3DFMT_X8R8G8B8, 0, D3DRTYPE_TEXTURE, (D3DFORMAT) '1ITA') == D3D_OK);
	bool supportsATI2N = (d3d->CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, D3DFMT_X8R8G8B8, 0, D3DRTYPE_TEXTURE, (D3DFORMAT) '2ITA') == D3D_OK);
	bool supportsDXT1  = (d3d->CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, D3DFMT_X8R8G8B8, 0, D3DRTYPE_TEXTURE, D3DFMT_DXT1) == D3D_OK);
	bool supportsDXT5  = (d3d->CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, D3DFMT_X8R8G8B8, 0, D3DRTYPE_TEXTURE, D3DFMT_DXT5) == D3D_OK);
	bool supportsRGBX8 = (d3d->CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, D3DFMT_X8R8G8B8, 0, D3DRTYPE_TEXTURE, D3DFMT_X8R8G8B8) == D3D_OK);

	if ((trilinearAniso = renderer->addSamplerState(TRILINEAR_ANISO, CLAMP, CLAMP, CLAMP)) == SS_NONE) return false;

	mode = -1;
	memset(modesSupported, 0, sizeof(modesSupported));
	if (supportsRGBX8){
		if ((baseRGB = renderer->addTexture("RoadRGB.htex", true, trilinearAniso)) == TEXTURE_NONE) return false;

		modesSupported[5] = true;
		mode = 5;
	}
	if (supportsDXT1){
		if ((baseDXT1  = renderer->addTexture("RoadDXT1.htex", true, trilinearAniso)) == TEXTURE_NONE) return false;

		modesSupported[0] = true;
		mode = 0;
		if (supportsDXT5){
			if ((baseYDXT1    = renderer->addTexture("RoadYDXT1.htex",    true, trilinearAniso)) == TEXTURE_NONE) return false;
			if ((baseCbCrDXT5 = renderer->addTexture("RoadCbCrDXT5.htex", true, trilinearAniso)) == TEXTURE_NONE) return false;

			modesSupported[1] = true;
			modesSupported[2] = true;
			mode = 2;
		}
	}
	if (supportsATI1N && supportsATI2N){
		if ((baseYATI1N    = renderer->addTexture("RoadYATI1N.htex",    true, trilinearAniso)) == TEXTURE_NONE) return false;
		if ((baseCbCrATI2N = renderer->addTexture("RoadCbCrATI2N.htex", true, trilinearAniso)) == TEXTURE_NONE) return false;

		modesSupported[3] = true;
		modesSupported[4] = true;
		mode = 4;
	}
	if (mode < 0){
		ErrorMsg("Not even RGB8 supported!\nGet a video card from this millenium!");
		return false;
	}

	if ((shader[0] = renderer->addShader("shader.shd")) == SHADER_NONE) return false;
	if ((shader[1] = renderer->addShader("shader.shd", "#define USE_YCBCR\n#define Y y\n#define CB y\n#define CR w\n")) == SHADER_NONE) return false;
	if ((shader[2] = renderer->addShader("shader.shd", "#define USE_YCBCR\n#define Y x\n#define CB x\n#define CR y\n")) == SHADER_NONE) return false;

	FormatDesc format[] = { 0, TYPE_VERTEX, FORMAT_FLOAT, 2 };
	if ((vertexFormat = renderer->addVertexFormat(format, elementsOf(format))) == VF_NONE) return false;

	return true;
}

void App::drawFrame(){
	float4x4 projection = toD3DProjection(perspectiveMatrixX(1.5f, width, height, 0.01f, 10));
	float4x4 modelview = rotateXY(-wx, -wy);
	modelview.translate(-camPos);
	float4x4 mvp = projection * modelview;


    dev->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_RGBA(50, 100, 75, 0), 1.0f, 0);

	int base1Unit = -1;

	renderer->reset();
	switch (mode){
	case 0:
		renderer->setShader(shader[0]);
		renderer->setTexture("Base0", baseDXT1, trilinearAniso);
		break;
	case 1:
		dev->SetSamplerState(base1Unit = ((Direct3DRenderer *) renderer)->getSamplerUnit(shader[1], "Base1"), D3DSAMP_MAXMIPLEVEL, 1);
		// Fall-through to case 2:
	case 2:
		renderer->setShader(shader[1]);
		renderer->setTexture("Base0", baseYDXT1, trilinearAniso);
		renderer->setTexture("Base1", baseCbCrDXT5, trilinearAniso);
		break;
	case 3:
		dev->SetSamplerState(base1Unit = ((Direct3DRenderer *) renderer)->getSamplerUnit(shader[2], "Base1"), D3DSAMP_MAXMIPLEVEL, 1);
		// Fall-through to case 4:
	case 4:
		renderer->setShader(shader[2]);
		renderer->setTexture("Base0", baseYATI1N, trilinearAniso);
		renderer->setTexture("Base1", baseCbCrATI2N, trilinearAniso);
		break;
	case 5:
		renderer->setShader(shader[0]);
		renderer->setTexture("Base0", baseRGB, trilinearAniso);
		break;
	}
	renderer->setVertexFormat(vertexFormat);
	renderer->setShaderConstant4x4f("mvp", mvp);
	renderer->apply();


	const float vertices[] = {
		-1,  1,
		 1,  1,
		 1, -1,
		-1, -1,
	};

	dev->DrawPrimitiveUP(D3DPT_TRIANGLEFAN, 2, vertices, sizeof(float2));

	if (base1Unit >= 0){
		dev->SetSamplerState(base1Unit, D3DSAMP_MAXMIPLEVEL, 0);
	}





	renderer->setup2DMode(0, (float) width, 0, (float) height);

	static char *modeNames[] = {
		"DXT1\n4 bpp",
		"YCbCr DXT\n4.5 bpp",
		"YCbCr DXT\n6 bpp",
		"YCbCr 3Dc+\n4.5 bpp",
		"YCbCr 3Dc+\n6 bpp",
		"RGB\n24 bpp (32 in practice)",
	};

	renderer->drawText(modeNames[mode], 10, height - 80.0f, 30, 36, defaultFont, linearClamp, blendSrcAlpha, noDepthTest);

	if (errorMsg){
		if (time < errorTime + 3){
			renderer->drawText(errorMsg, 0.5f * (width - 30 * renderer->getTextWidth(defaultFont, errorMsg)), 0.5f * height, 30, 36, defaultFont, linearClamp, blendSrcAlpha, noDepthTest);
		} else {
			errorMsg = NULL;
		}
	}
}
